package core

import (
	"crypto/tls"
	"encoding/json"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"time"
)

// messages
var messages []Message

// Message
// 单个消息
type Message struct {
	Role    string `json:"role"`
	Content string `json:"content"`
}

// Payload
// 请求数据
type Payload struct {
	Model    string    `json:"model"`
	Messages []Message `json:"messages"`
}

// AnswerItem
// 回答数据
type AnswerItem struct {
	FinishReason string  `json:"finish_reason"`
	Index        int     `json:"index"`
	Message      Message `json:"message"`
}

// Answer
// 回答
type Answer struct {
	Id      string       `json:"id"`
	Object  int          `json:"object"`
	Created interface{}  `json:"created"`
	Model   string       `json:"model"`
	Usage   interface{}  `json:"usage"`
	Choices []AnswerItem `json:"choices"`
}

// GPT
type GPT struct {
	ApiKey  string
	Proxy   string
	Timeout int
}

// BuildPayload
// 构建请求数据
//
//	@receiver g
//	@param question
//	@return string
func (g *GPT) BuildPayload(question string) string {
	messages = append(messages, Message{
		Role:    "user",
		Content: question,
	})

	payload := Payload{
		Model:    "gpt-3.5-turbo",
		Messages: messages,
	}

	str, _ := json.Marshal(&payload)
	return string(str)
}

// HttpClientWithProxy
// 返回Http客户端对象
//
//	@receiver g
//	@param proxy
//	@return *http.Client
func (g *GPT) HttpClientWithProxy(proxy string) *http.Client {
	transport := &http.Transport{
		TLSClientConfig: &tls.Config{
			InsecureSkipVerify: true,
		},
	}
	if proxy != "" {
		proxyAddr, _ := url.Parse(proxy)
		transport.Proxy = http.ProxyURL(proxyAddr)
	}

	timeout := 10
	if g.Timeout > 0 {
		timeout = g.Timeout
	}

	return &http.Client{
		Timeout:   time.Duration(timeout) * time.Second,
		Transport: transport,
	}
}

// GetAnswer
// 发起请求
//
//	@receiver g
//	@param question
//	@return Answer
func (g *GPT) GetAnswer(question string) (Answer, error) {
	payload := g.BuildPayload(question)

	api := "https://api.openai.com/v1/chat/completions"
	req, _ := http.NewRequest("POST", api, strings.NewReader(payload))
	req.Header.Set("Content-Type", "application/json")
	req.Header.Set("Authorization", "Bearer "+g.ApiKey)

	asr := Answer{}

	client := g.HttpClientWithProxy(g.Proxy)
	resp, err := client.Do(req)
	if err != nil {
		return asr, err
	}
	body, _ := ioutil.ReadAll(resp.Body)

	json.Unmarshal(body, &asr)

	return asr, nil
}
