package core

import (
	"chatgpt/utils"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"strconv"
	"sync"
)

// RequestData
type RequestData struct {
	Words string `json:"words"`
	Time  int    `json:"time"`
	Sign  string `json:"sign"`
}

// Response
// 响应体
type Response struct {
	Code int    `json:"code"`
	Msg  string `json:"msg"`
	Data string `json:"data"`
	Sign string `json:"sign"`
}

// ResponseData
// 响应数据
type ResponseData map[string]interface{}

// BuildResponse
// 构建响应体
//
//	@param privateKey
//	@param code
//	@param msg
//	@param data
//	@return string
func BuildResponse(privateKey string, code int, msg string, data ResponseData) string {
	resp := Response{
		Code: code,
		Msg:  msg,
	}
	dataJson, _ := json.Marshal(data)
	resp.Data = utils.Base64{
		Content: []byte(dataJson),
	}.Encode()
	if privateKey != "" {
		resp.Sign = utils.MakeSign(resp, privateKey)
	}

	respJson, _ := json.Marshal(resp)

	return string(respJson)
}

var wg sync.WaitGroup

// proxyAddr
// 代理配置
var proxyAddr string = utils.GetProxyServer()

// apiKey
// GPT Api Key
var apiKey string = utils.Conf.Get("gpt", "api_key")

// privateKey
// 签名密钥
var privateKey string = utils.Conf.Get("init", "private_key")

// TimeoutValue
// 超时配置
var TimeoutValue string = utils.Conf.Get("init", "timeout")

// Action
// 处理请求
//
//	@param w
//	@param r
func Handler(w http.ResponseWriter, r *http.Request) {
	w.Header().Add("Access-Control-Allow-Origin", "*")
	w.Header().Add("Access-Control-Allow-Methods", "POST, GET, PUT, DELETE, OPTIONS")
	w.Header().Add("Content-Type", "application/json")
	Timeout, _ := strconv.Atoi(TimeoutValue)
	if r.Method == "POST" {
		var requestData RequestData
		raw, _ := ioutil.ReadAll(r.Body)
		json.Unmarshal(raw, &requestData)

		if privateKey != "" && requestData.Sign != utils.MakeSign(requestData, privateKey) {
			fmt.Fprintln(w, BuildResponse(privateKey, 0, "sign error", ResponseData{}))
			return
		}

		question := utils.Base64{
			Content: []byte(requestData.Words),
		}.Decode()
		if len(question) > 0 {
			gpt := &GPT{
				ApiKey:  apiKey,
				Proxy:   proxyAddr,
				Timeout: Timeout,
			}
			wg.Add(1)
			go func() {
				defer wg.Done()
				if result, err := gpt.GetAnswer(question); err == nil {
					fmt.Fprintln(w, BuildResponse(privateKey, 1, "success", ResponseData{
						"answer": result.Choices,
					}))
				}
			}()
			wg.Wait()
			return
		}
	}
	fmt.Fprintln(w, BuildResponse(privateKey, 0, "error", ResponseData{}))
}
